<?php
/**
 * @file classes for tfa_totp
 */

/**
 * Class TfaTotp
 */
class Tfaduo extends TfaBasePlugin implements TfaValidationPluginInterface {
  protected $host;
  protected $skey;
  protected $ikey;

  public function __construct(array $context) {
    parent::__construct($context);
    $this->host = variable_get('tfa_duo_host', '');
    $this->skey = variable_get('tfa_duo_skey', '');
    $this->ikey = variable_get('tfa_duo_ikey', '');
  }

  /**
   * @copydoc TfaValidationPluginInterface::getForm()
   */
  public function getForm(array $form, array &$form_state) {
    $account = user_load($this->context['uid']);
    $dev = $this->getMethods($account);
    if (isset($dev['enroll'])) {
      $form['enroll'] = array('#markup' => l("You must enroll by going here, then login again", $dev['enroll'], array('attributes' => array('target' => '_blank'))));
      return $form;
    }
    $meth = array();
    //todo get each device and only show options for the device
    if (count($dev) > 1) {
      $meth['push'] = t('Push using the above device');
      $meth['passcode'] = t('Use a passcode');
      $meth['phone'] = t('Call the above device');
      $meth['sms'] = t('Send a new batch of passcodes');
      $form['device'] = array(
        '#type' => 'select',
        '#options' => $dev,
        '#title' => t('What device would you like to use for authentication?')
      );
    } else {
      $meth['passcode'] = t('Use a passcode');

    }

    $form['meth'] = array(
      '#type' => 'select',
      '#options' => $meth,
      '#title' => t('How would you like to authenticate?')
    );
    $form['pass'] = array('#type' => 'textfield', '#title' => 'For passcode options, enter your passcode here');
    $form['submit'] = array('#type' => 'submit', '#value' => 'submit');
    return $form;
  }

  /**
   * @copydoc TfaValidationPluginInterface::getForm()
   */
  public function validateForm(array $form, array &$form_state) {
    $res = $this->duoValidate($form_state['values']['meth'], $form_state['values']['device'], $form_state['values']['pass']);
    if ($res['result'] == 'allow') {
      $form_state['storage']['allow'] = 1;
      drupal_set_message($res['msg']);
    }
    else {
      form_set_error('', $res['msg']);
    }
  }

  /**
   * @copydoc TfaBasePlugin::submitForm()
   */
  public function submitForm(array $form, array &$form_state) {
    if ($form_state['storage']['allow'] == 1) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * This will build a request and return a response from Duo
   * @param $method GET/POST
   * @param $params Variables
   * @param $path Method on Duo's side
   * @return mixed normally json
   */
  public function getDuo_response($method, $params, $path) {
    $date = date('r');
    $canon = join("\n", array(
      $date,
      strtoupper($method),
      strtolower($this->host),
      $path,
      $this->canonicalize_params($params)
    ));
    $ch = curl_init();
    $sig = hash_hmac('sha1', $canon, $this->skey, FALSE);
    $auth = join(':', array($this->ikey, $sig));
    $auth = 'Basic ' . base64_encode($auth);
    $url = 'https://' . $this->host . $path;
    curl_setopt($ch, CURLOPT_URL, $url);
    $headers = array();
    $headers[] = 'Authorization: ' . $auth;
    $headers[] = 'Date: ' . $date;
    $headers[] = 'Host: ' . $this->host;
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $this->canonicalize_params($params));
    $res = curl_exec($ch);
    curl_close($ch);
    return $res;
  }

  /**
   * Setup params for a web request
   * @param $params
   * @return string
   */
  private function canonicalize_params($params) {
    $items = array();
    foreach ($params as $key => $val) {
      $items[] = "$key=$val";
    }
    sort($items);
    $cannon = join('&', $items);
    return $cannon;
  }

  /**
   * returns the methods that can be used by a user from duo
   * @param $account
   * @return array|bool
   */
  private function getMethods($account) {
    $dev = array();


    $pre = json_decode($this->getDuo_response('POST', array('username' => $account->name), '/auth/v2/preauth'));
    if ($pre->response->result == 'enroll') {
      $device['enroll'] = $pre->response->enroll_portal_url;
      return $device;
    }
    if ($pre->response->status_msg == 'Account in bypass mode') {
      $dev['bypass'] = t('bypass mode');
      return $dev;
    }

    if ($pre->response->status_msg != 'Account is active') {
      return FALSE;
    }

    foreach ($pre->response->devices as $key => $device) {
      if ($device->type == 'phone') {
        $dev[$device->device] = $device->display_name;
      }
    }
    return $dev;
  }

  /**
   * Validates a Duo request
   * @param $meth
   * @param $device
   * @param $code
   * @return array
   */
  private function duoValidate($meth, $device, $code) {
    $account = user_load($this->context['uid']);
    if ($device == 'bypass') {
      $return['msg'] = 'account in bypass mode';
      $return['result'] = 'allow';
      return $return;
    }
    if ($meth == 'push') {
      $res = $this->getDuo_response('POST', array(
        'username' => $account->name,
        'factor' => 'push',
        'ipaddr' => ip_address(),
        'device' => $device
      ), '/auth/v2/auth');
    }
    if ($meth == 'passcode') {
      $res = $this->getDuo_response('POST', array(
        'username' => $account->name,
        'factor' => 'passcode',
        'ipaddr' => ip_address(),
        'passcode' => $code
      ), '/auth/v2/auth');
    }
    if ($meth == 'phone') {
      $res = $this->getDuo_response('POST', array(
        'username' => $account->name,
        'factor' => 'phone',
        'ipaddr' => ip_address(),
        'device' => $device
      ), '/auth/v2/auth');
    }


    $return = array();

    if (!isset($res)) {
      $return['msg'] = 'No valid options';
      $return['result'] = 'FAIL';
    }
    else {
      $res = json_decode($res);
      $res = $res->response;
      $return['msg'] = $res->status_msg;
      $return['result'] = $res->result;
    }
    return $return;
  }


  /**
   * @copydoc TfaBasePlugin::ready()
   */

  public function ready() {
    if ($this->ikey == '') {
      return FALSE;
    }
    if (user_access('require duo')) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }
}


